# -*- coding: utf-8 -*-
#########################################################################
#
# Copyright 2015-2018, GeoSolutions Sas.
# All rights reserved.
#
# This source code is licensed under the BSD-style license found in the
# LICENSE file in the root directory of this source tree.
#
#########################################################################
from django import template

register = template.Library()


@register.inclusion_tag(
    'geonode-mapstore-client/_client_composer_js.html', takes_context=True)
def client_composer_js(context, options=None):
    """
    React Client Composer js template tag.
    """
    return context


@register.inclusion_tag(
    'geonode-mapstore-client/_client_viewer_js.html', takes_context=True)
def client_viewer_js(context, options=None):
    """
    React client viewer js template tag.
    """
    return context



# -*- coding: utf-8 -*-
#########################################################################
#
# Copyright 2015-2018, GeoSolutions Sas.
# All rights reserved.
#
# This source code is licensed under the BSD-style license found in the
# LICENSE file in the root directory of this source tree.
#
#########################################################################
from django.apps import AppConfig as BaseAppConfig


class AppConfig(BaseAppConfig):

    name = "geonode_mapstore_client"
    label = "geonode_mapstore_client"

    def ready(self):
        super(AppConfig, self).ready()
        # run_setup_hooks()



# -*- coding: utf-8 -*-
#########################################################################
#
# Copyright 2015-2018, GeoSolutions Sas.
# All rights reserved.
#
# This source code is licensed under the BSD-style license found in the
# LICENSE file in the root directory of this source tree.
#
#########################################################################
import logging
import traceback

from django.conf import settings

logger = logging.getLogger("geonode_mapstore_client.settings")

DEFAULT_MS2_BACKGROUNDS = [{
        "type": "osm",
        "title": "Open Street Map",
        "name": "mapnik",
        "source": "osm",
        "group": "background",
        "visibility": True
    },
    {
        "type": "google",
        "title": "Google HYBRID",
        "name": "HYBRID",
        "source": "google",
        "group": "background",
        "visibility": False
    },
    {
        "type": "mapquest",
        "title": "MapQuest OSM",
        "name": "osm",
        "source": "mapquest",
        "group": "background",
        "visibility": False
    },
    {
        "type": "bing",
        "title": "Bing Aerial",
        "apiKey": "AqTGBsziZHIJYYxgivLBf0hVdrAk9mWO5cQcb8Yux8sW5M8c8opEC2lZqKR1ZZXf",
        "name": "Aerial",
        "source": "bing",
        "group": "background",
        "visibility": False
    },
    {
        "type": "tileprovider",
        "title": "NASAGIBS Night 2012",
        "provider": "NASAGIBS.ViirsEarthAtNight2012",
        "name": "Night2012",
        "source": "nasagibs",
        "group": "background",
        "visibility": False
    },
    {
        "type": "wms",
        "url": "http://www.realvista.it/reflector/open/service",
        "visibility": False,
        "title": "e-Geos Ortofoto RealVista 1.0",
        "name": "rv1",
        "group": "background",
        "format": "image/jpeg"
    },
    {
        "type": "wms",
        "url": "https://demo.geo-solutions.it/geoserver/wms",
        "visibility": False,
        "title": "Natural Earth",
        "name": "sde:NE2_HR_LC_SR_W_DR",
        "group": "background",
        "format": "image/png"
    },
    {
        "type": "wms",
        "url": "https://demo.geo-solutions.it/geoserver/wms",
        "visibility": False,
        "title": "Hypsometric",
        "name": "sde:HYP_HR_SR_OB_DR",
        "group": "background",
        "format": "image/png"
    },
    {
        "type": "wms",
        "url": "https://demo.geo-solutions.it/geoserver/wms",
        "visibility": False,
        "title": "Gray Earth",
        "name": "sde:GRAY_HR_SR_OB_DR",
        "group": "background",
        "format": "image/png"
    },
    {
        "type": "tileprovider",
        "title": "OpenTopoMap",
        "provider": "OpenTopoMap",
        "name": "OpenTopoMap",
        "source": "OpenTopoMap",
        "group": "background",
        "visibility": False
    }
]

MAP_BASELAYERS = getattr(settings, "MAPSTORE_BASELAYERS", DEFAULT_MS2_BACKGROUNDS)


def get_geonode_catalogue_service():
    pycsw = getattr(settings, "PYCSW", None)
    if pycsw:
        try:
            pycsw_config = pycsw["CONFIGURATION"]
            if pycsw_config:
                    pycsw_catalogue = {
                        ("%s" % pycsw_config['metadata:main']['identification_title']): {
                            "url": settings.CATALOGUE['default']['URL'],
                            "type": "csw",
                            "title": pycsw_config['metadata:main']['identification_title'],
                            "autoload": True
                         }
                    }

                    return pycsw_catalogue
        except:
            traceback.print_exc()
            tb = traceback.format_exc()
            logger.error(tb)

    return None

GEONODE_CATALOGUE_SERVICE = get_geonode_catalogue_service()

DEFAULT_MS2_CATALOGUE_SERVICES = {
    "Demo WMS Service": {
        "url": "https://demo.geo-solutions.it/geoserver/wms",
        "type": "wms",
        "title": "Demo WMS Service",
        "autoload": False
     },
    "Demo WMTS Service": {
        "url": "https://demo.geo-solutions.it/geoserver/gwc/service/wmts",
        "type": "wmts",
        "title": "Demo WMTS Service",
        "autoload": False
    }
}
DEFAULT_MS2_CATALOGUE_SELECTED_SERVICE = GEONODE_CATALOGUE_SERVICE.keys()[0] if GEONODE_CATALOGUE_SERVICE else "Demo WMS Service"

CATALOGUE_SERVICES = getattr(settings, "MAPSTORE_CATALOGUE_SERVICES", DEFAULT_MS2_CATALOGUE_SERVICES)
CATALOGUE_SELECTED_SERVICE = getattr(settings, "MAPSTORE_CATALOGUE_SELECTED_SERVICE", DEFAULT_MS2_CATALOGUE_SELECTED_SERVICE)



# -*- coding: utf-8 -*-
#########################################################################
#
# Copyright 2015-2018, GeoSolutions Sas.
# All rights reserved.
#
# This source code is licensed under the BSD-style license found in the
# LICENSE file in the root directory of this source tree.
#
#########################################################################
import json
import math
import logging
import traceback

from .utils import GoogleZoom
from .settings import (MAP_BASELAYERS,
                       GEONODE_CATALOGUE_SERVICE,
                       CATALOGUE_SERVICES,
                       CATALOGUE_SELECTED_SERVICE)

from django.core.serializers.json import DjangoJSONEncoder

from django.contrib.gis.geos import Polygon
from django.contrib.gis.gdal import SpatialReference, CoordTransform

logger = logging.getLogger("geonode_mapstore_client.converters")


class MapStore2GeoExtConfigConverter(object):

    def convert(self, viewer, request):
        data = {}
        data['version'] = 2
        data['widgetsConfig'] = {}  # TODO

        try:
            viewer_obj = json.loads(viewer)

            # Map Definition
            ms2_map = {}
            ms2_map['projection'] = viewer_obj['map']['projection']
            ms2_map['units'] = viewer_obj['map']['units']
            ms2_map['zoom'] = viewer_obj['map']['zoom']
            ms2_map['maxExtent'] = viewer_obj['map']['maxExtent']
            ms2_map['maxResolution'] = viewer_obj['map']['maxResolution']

            # Backgrouns
            ms2_map['layers'] = MAP_BASELAYERS + [
                # TODO: covnert Viewer Backgroun Layers
                # Add here more backgrounds e.g.:
    			# {
    			# 	"type": "wms",
    			# 	"url": "https://demo.geo-solutions.it/geoserver/wms",
    			# 	"visibility": True,
    			# 	"opacity": 0.5,
    			# 	"title": "Weather data",
    			# 	"name": "nurc:Arc_Sample",
    			# 	"group": "Meteo",
    			# 	"format": "image/png",
    			# 	"bbox": {
    			# 		"bounds": {
    			# 			"minx": -25.6640625,
    			# 			"miny": 26.194876675795218,
    			# 			"maxx": 48.1640625,
    			# 			"maxy": 56.80087831233043
    			# 		},
    			# 		"crs": "EPSG:4326"
    			# 	}
    			# }, ...
            ]

            # Overlays
            overlays, selected = self.get_overlays(viewer)

            if selected:
                center, zoom = self.get_center_and_zoom(viewer_obj['map'], selected)
                ms2_map['center'] = center
                ms2_map['zoom'] = zoom
            else:
                ms2_map['center'] = {
                    "x": self.get_valid_number(viewer_obj['map']['center'][0]),
    			    "y": self.get_valid_number(viewer_obj['map']['center'][1]),
    			    "crs": viewer_obj['map']['projection']
                }

            for overlay in overlays:
                ms2_map['layers'].append(overlay)

            data['map'] = ms2_map

            # Catalogue Services Definition
            ms2_catalogue = {}
            ms2_catalogue['selectedService'] = CATALOGUE_SELECTED_SERVICE
            if GEONODE_CATALOGUE_SERVICE:
                ms2_catalogue['services'] = dict(CATALOGUE_SERVICES.items() + GEONODE_CATALOGUE_SERVICE.items())
            else:
                ms2_catalogue['services'] = CATALOGUE_SERVICES

            data['catalogServices'] = ms2_catalogue
        except:
            traceback.print_exc()
            tb = traceback.format_exc()
            logger.error(tb)

        return json.dumps(data, cls=DjangoJSONEncoder, sort_keys=True)

    def get_valid_number(self, number, default=None, complementar=False):
        x = float(number)
        is_nan = math.isnan(x)
        is_inf = math.isinf(x)
        if not is_nan and not is_inf:
            return x
        elif default:
            return default if not complementar else -default
        return 0

    def get_overlays(self, viewer):
        overlays = []
        selected = None
        try:
            viewer_obj = json.loads(viewer)
            layers = viewer_obj['map']['layers']
            sources = viewer_obj['sources']

            for layer in layers:
                if 'group' not in layer or layer['group'] != "background":
                    source = sources[layer['source']]
                    if 'ptype' in source and 'gxp_wmscsource' == source['ptype']:
                        overlay = {}
                        overlay['type'] = "wms"
                        overlay['url'] = source['url']
                        overlay['visibility'] = layer['visibility'] if 'visibility' in layer else True
                        overlay['opacity'] = layer['opacity'] if 'opacity' in layer else 1.0
                        overlay['title'] = layer['title'] if 'title' in layer else ''
                        overlay['name'] = layer['name'] if 'name' in layer else ''
                        overlay['group'] = layer['group'] if 'group' in layer else ''
                        overlay['format'] = layer['format'] if 'format' in layer else "image/png"

                        overlay['bbox'] = {}
                        if 'capability' in layer:
                            capa = layer['capability']
                            if 'styles' in capa:
                                overlay['styles'] = capa['styles']
                            if 'abstract' in capa:
                                overlay['abstract'] = capa['abstract']
                            if 'attribution' in capa:
                                overlay['attribution'] = capa['attribution']
                            if 'keywords' in capa:
                                overlay['keywords'] = capa['keywords']
                            if 'dimensions' in capa:
                                overlay['dimensions'] = capa['dimensions']
                            if 'llbbox' in capa:
                                overlay['llbbox'] = capa['llbbox']
                            if 'bbox' in capa:
                                bbox = capa['bbox']
                                if viewer_obj['map']['projection'] in bbox:
                                    proj = viewer_obj['map']['projection']
                                    bbox = capa['bbox'][proj]
                                    overlay['bbox']['bounds'] = {
                						"minx": self.get_valid_number(bbox['bbox'][0]),
                						"miny": self.get_valid_number(bbox['bbox'][1]),
                						"maxx": self.get_valid_number(bbox['bbox'][2]),
                						"maxy": self.get_valid_number(bbox['bbox'][3])
                					}
                                    overlay['bbox']['crs'] = bbox['srs']

                        if 'bbox' in layer and not overlay['bbox']:
                            overlay['bbox']['bounds'] = {
        						"minx": self.get_valid_number(layer['bbox'][0], default=layer['bbox'][2], complementar=True),
        						"miny": self.get_valid_number(layer['bbox'][1], default=layer['bbox'][3], complementar=True),
        						"maxx": self.get_valid_number(layer['bbox'][2], default=layer['bbox'][0], complementar=True),
        						"maxy": self.get_valid_number(layer['bbox'][3], default=layer['bbox'][1], complementar=True)
        					}
                            overlay['bbox']['crs'] = layer['srs'] if 'srs' in layer else viewer_obj['map']['projection']

                    overlays.append(overlay)
                    if not selected or ('selected' in layer and layer['selected']):
                        selected = overlay
        except:
            traceback.print_exc()
            tb = traceback.format_exc()
            logger.error(tb)

        return (overlays, selected)

    def get_center_and_zoom(self, view_map, overlay):
        center = {
            "x": self.get_valid_number(overlay['bbox']['bounds']['minx'] +
                                       (overlay['bbox']['bounds']['maxx'] -
                                        overlay['bbox']['bounds']['minx']) / 2),
            "y": self.get_valid_number(overlay['bbox']['bounds']['miny'] +
                                       (overlay['bbox']['bounds']['maxy'] -
                                        overlay['bbox']['bounds']['miny']) / 2),
            "crs": overlay['bbox']['crs']
        }
        zoom = view_map['zoom']
        # max_extent = view_map['maxExtent']
        # map_crs = view_map['projection']
        try:
            ov_bbox = [self.get_valid_number(overlay['bbox']['bounds']['minx']),
                       self.get_valid_number(overlay['bbox']['bounds']['miny']),
                       self.get_valid_number(overlay['bbox']['bounds']['maxx']),
                       self.get_valid_number(overlay['bbox']['bounds']['maxy']),]
            ov_crs = overlay['bbox']['crs']
            srid = int(ov_crs.split(':')[1])
            srid = 3857 if srid == 900913 else srid
            poly = Polygon((
                (ov_bbox[0], ov_bbox[1]),
                (ov_bbox[0], ov_bbox[3]),
                (ov_bbox[2], ov_bbox[3]),
                (ov_bbox[2], ov_bbox[1]),
                (ov_bbox[0], ov_bbox[1])), srid=srid)
            gcoord = SpatialReference(4326)
            ycoord = SpatialReference(srid)
            trans = CoordTransform(ycoord, gcoord)
            poly.transform(trans)
            zoom = GoogleZoom().get_zoom(poly) + 1
        except:
            traceback.print_exc()
            tb = traceback.format_exc()
            logger.error(tb)

        return (center, zoom)


ms2_config_converter = MapStore2GeoExtConfigConverter()



# -*- coding: utf-8 -*-
#########################################################################
#
# Copyright 2015-2018, GeoSolutions Sas.
# All rights reserved.
#
# This source code is licensed under the BSD-style license found in the
# LICENSE file in the root directory of this source tree.
#
#########################################################################
from geonode.client.hooksets import GeoExtHookSet
from .converters import ms2_config_converter


class MapStoreHookSet(GeoExtHookSet):

    def get_request(self, context):
        if context and 'request' in context:
            return context['request']
        return None

    def get_access_token(self, request):
        if request and 'access_token' in request.session:
            return request.session['access_token']
        return None

    def initialize_context(self, context):
        if context:
            request = self.get_request(context)
            context['ACCESS_TOKEN'] = self.get_access_token(request)
            config = context['viewer'] if 'viewer' in context else None
            if not config and len(context.dicts) > 0:
                for ctx in context.dicts:
                    if 'preview' in ctx and (
                        ctx['preview'] in ('geoext', 'mapstore', 'mapstore2', 'ms2') and 'config' in ctx):
                        config = ctx['config']
            ms2_config = {}
            if config:
                try:
                    ms2_config = ms2_config_converter.convert(config, request)
                except:
                    ms2_config = '{}'
            context['ms2_config'] = ms2_config

    # Layers
    def layer_detail_template(self, context=None):
        self.initialize_context(context)
        return 'geonode-mapstore-client/layer_map.html'

    def layer_new_template(self, context=None):
        self.initialize_context(context)
        return 'geonode-mapstore-client/layer_map.html'

    def layer_view_template(self, context=None):
        self.initialize_context(context)
        return 'geonode-mapstore-client/layer_map.html'

    # -- Not implemented yet
    # def layer_edit_template(self, context=None):
    #     self.initialize_context(context)
    #     return 'geonode-mapstore-client/layer_map.html'

    def layer_update_template(self, context=None):
        self.initialize_context(context)
        return 'geonode-mapstore-client/layer_map.html'

    # -- Not implemented yet
    # def layer_embed_template(self, context=None):
    #     self.initialize_context(context)
    #     return 'geonode-mapstore-client/layer_map.html'

    def layer_download_template(self, context=None):
        self.initialize_context(context)
        return 'geonode-mapstore-client/layer_map.html'

    # Maps
    def map_detail_template(self, context=None):
        self.initialize_context(context)
        return 'geonode-mapstore-client/map_detail.html'

    # -- Not implemented yet
    # def map_new_template(self, context=None):
    #     self.initialize_context(context)
    #     return 'geonode-mapstore-client/map_new.html'
    #
    # def map_view_template(self, context=None):
    #     self.initialize_context(context)
    #     return 'geonode-mapstore-client/map_view.html'
    #
    # def map_edit_template(self, context=None):
    #     self.initialize_context(context)
    #     return 'geonode-mapstore-client/edit_map.html'
    #
    # def map_update_template(self, context=None):
    #     self.initialize_context(context)
    #     return 'geonode-mapstore-client/edit_map.html'
    #
    # def map_embed_template(self, context=None):
    #     self.initialize_context(context)
    #     return 'geonode-mapstore-client/map_view.html'
    #
    # def map_download_template(self, context=None):
    #     self.initialize_context(context)
    #     return 'geonode-mapstore-client/map_view.html'



# -*- coding: utf-8 -*-
#########################################################################
#
# Copyright 2015-2018, GeoSolutions Sas.
# All rights reserved.
#
# This source code is licensed under the BSD-style license found in the
# LICENSE file in the root directory of this source tree.
#
#########################################################################
default_app_config = "geonode_mapstore_client.apps.AppConfig"


class GeoNodeMapstore2Exception(Exception):
    """Base class for exceptions in this module."""
    pass



import os
from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(here, 'VERSION')) as version_file:
        version = version_file.read().strip()

setup(
    name='django-geonode-mapstore-client',
    version=version,
    author='Alessio Fabiani',
    author_email='alessio.fabiani@gmail.com',
    url='https://github.com/GeoNode/geonode-mapstore-client',
    description="Use GeoNode client in your django projects",
    long_description=open(os.path.join(here, 'README.md')).read(),
    license='BSD, see LICENSE file.',
    install_requires=[],
    packages=find_packages(),
    include_package_data = True,
    zip_safe = False,
    classifiers  = [],
)
